from __future__ import annotations

from core.constants import CellValue, checkCellValue
from .SetLayerValueCommand import SetLayerValueCommand
from ..Logic import Logic


class SetImpassableValueCommand(SetLayerValueCommand):

    def check(self, logic: Logic) -> bool:
        value = self._value
        if not checkCellValue("impassable", value):
            return False
        cell = self._cell
        world = logic.world
        if not world.contains(cell):
            return False

        value = self._value
        impassableValue = world.impassable.getValue(cell)
        if value == CellValue.NONE:
            if impassableValue == CellValue.NONE:  # Value already removed
                return False
        else:
            if impassableValue != CellValue.NONE:  # Value already set
                return False
            if world.ground.getValue(cell) == CellValue.GROUND_SEA:
                return False
            if world.objects.getValue(cell) != CellValue.NONE:
                return False

        return True
        
    def execute(self, logic: Logic):
        cell = self._cell
        value = self._value
        world = logic.world
        world.impassable.setValue(cell, value)
        world.impassable.notifyCellChanged(cell)

        if self._fill:
            x, y = cell[0], cell[1]
            logic.addCommand(SetImpassableValueCommand((x + 1, y), value, True))
            logic.addCommand(SetImpassableValueCommand((x - 1, y), value, True))
            logic.addCommand(SetImpassableValueCommand((x, y + 1), value, True))
            logic.addCommand(SetImpassableValueCommand((x, y - 1), value, True))

