from __future__ import annotations

from core.constants import CellValue, checkCellValue
from .SetLayerValueCommand import SetLayerValueCommand
from ..Logic import Logic


class SetObjectsValueCommand(SetLayerValueCommand):

    def check(self, logic: Logic) -> bool:
        value = self._value
        if not checkCellValue("objects", value):
            return False
        cell = self._cell
        world = logic.world
        if not world.contains(cell):
            return False
        
        objectsValue = world.objects.getValue(cell)
        if value == CellValue.NONE:
            if objectsValue == CellValue.NONE:  # Value already removed
                return False
        else:
            if objectsValue != CellValue.NONE:  # Value already set
                return False
            if world.ground.getValue(cell) == CellValue.GROUND_SEA:  # Can't build on the sea
                return False
            if world.impassable.getValue(cell) != CellValue.NONE:  # Can't build over a mountain
                return False

        return True

    def execute(self, logic: Logic, simulate: bool = False):
        cell = self._cell
        value = self._value
        world = logic.world
        world.objects.setValue(cell, value)
        world.objects.notifyCellChanged(cell)

        if self._fill:
            x, y = cell[0], cell[1]
            logic.addCommand(SetObjectsValueCommand((x + 1, y), value, True))
            logic.addCommand(SetObjectsValueCommand((x - 1, y), value, True))
            logic.addCommand(SetObjectsValueCommand((x, y + 1), value, True))
            logic.addCommand(SetObjectsValueCommand((x, y - 1), value, True))

