from typing import Tuple

from .ILayerListener import ILayerListener
from ..Listenable import Listenable
from ..constants.CellValue import CellValue


class Layer(Listenable[ILayerListener]):
    def __init__(self, size: Tuple[int, int], defaultValue: CellValue):
        super().__init__()
        self.__size = size
        self.__cells = []
        for y in range(size[1]):
            row = [defaultValue] * size[0]
            self.__cells.append(row)

    @property
    def size(self) -> Tuple[int, int]:
        return self.__size

    @property
    def width(self) -> int:
        return self.__size[0]

    @property
    def height(self) -> int:
        return self.__size[1]

    def __getitem__(self, coords: Tuple[int, int]) -> CellValue:
        return self.getValue(coords)

    def getValue(self, coords: Tuple[int, int]) -> CellValue:
        x, y = coords[0], coords[1]
        assert 0 <= x < self.__size[0], f"Invalid x={x}"
        assert 0 <= y < self.__size[1], f"Invalid y={y}"
        return self.__cells[y][x]

    def __setitem__(self, coords: Tuple[int, int], value: CellValue):
        self.setValue(coords, value)

    def setValue(self, coords: Tuple[int, int], value: CellValue):
        x, y = coords[0], coords[1]
        assert 0 <= x < self.__size[0], f"Invalid x={x}"
        assert 0 <= y < self.__size[1], f"Invalid y={y}"
        self.__cells[y][x] = value

    def fill(self, value: CellValue):
        for y in range(0, self.height):
            for x in range(0, self.width):
                self.__cells[y][x] = value

    # Layer listener

    def notifyCellChanged(self, cell: Tuple[int, int]):
        for listener in self.listeners:
            listener.cellChanged(self, cell)
