from abc import abstractmethod

from pygame import Surface
from typing import Tuple

from core import Listenable
from .IComponentListener import IComponentListener
from ..Mouse import Mouse
from ..theme.Theme import Theme


class Component(Listenable[IComponentListener]):
    def __init__(self, theme: Theme):
        super().__init__()
        self.__theme = theme

    @property
    def theme(self) -> Theme:
        return self.__theme

    # Component

    def dispose(self):
        pass

    def needRefresh(self) -> bool:
        return True

    @abstractmethod
    def render(self, surface: Surface):
        raise NotImplementedError()

    # Component Listener

    def notifyWorldCellClicked(self, cell: Tuple[int, int], mouse: Mouse):
        for listener in self.listeners:
            listener.worldCellClicked(cell, mouse)

    def notifyWorldCellEntered(self, cell: Tuple[int, int], mouse: Mouse, dragging: bool):
        for listener in self.listeners:
            listener.worldCellEntered(cell, mouse, dragging)
