from typing import Tuple

from pygame.surface import Surface

from core.constants import CellValue
from core.state import World, ILayerListener, Layer
from ui.component.Component import Component
from ui.theme.Theme import Theme


class LayerComponent(Component, ILayerListener):
    def __init__(self, theme: Theme, world: World, name: str):
        super().__init__(theme)
        self.__layer = world.getLayer(name)
        self.__tileset = theme.getTileset(name)
        self.__needRefresh = True

        # Listener
        self.__layer.registerListener(self)

    # Component

    def dispose(self):
        super().dispose()
        self.__layer.removeListener(self)

    def needRefresh(self) -> bool:
        return self.__needRefresh

    def render(self, surface: Surface):
        tileset = self.__tileset.getSurface()
        tileWidth, tileHeight = self.__tileset.tileSize
        tilesRect = self.__tileset.getTilesRect()
        for y in range(self.__layer.height):
            for x in range(self.__layer.width):
                value = self.__layer[x, y]
                if value == CellValue.NONE:
                    continue
                tileRect = tilesRect[value]
                tileCoords = (x * tileWidth, y * tileHeight)
                surface.blit(tileset, tileCoords, tileRect)
        self.__needRefresh = False

    # Layer listener

    def cellChanged(self, layer: Layer, cell: Tuple[int, int]):
        if layer == self.__layer:
            self.__needRefresh = True
