from __future__ import annotations
from typing import Dict, Tuple, TYPE_CHECKING

from pygame.rect import Rect
from pygame.surface import Surface

if TYPE_CHECKING:
    from .Theme import Theme


class Tileset:
    def __init__(self, theme: Theme, tileSize: Tuple[int, int], imageFile: str):
        self.__theme = theme
        self.__tileSize = tileSize
        self.__imageFile = imageFile
        self.__tilesRect: Dict[int, Rect] = {}

    @property
    def tileSize(self) -> Tuple[int, int]:
        return self.__tileSize

    def getSurface(self) -> Surface:
        return self.__theme.getSurface(self.__imageFile)

    def addTile(self, value: int, coords: Tuple[int, int]):
        self.__tilesRect[value] = Rect(
                coords[0] * self.__tileSize[0],
                coords[1] * self.__tileSize[1],
                self.__tileSize[0], self.__tileSize[1]
            )

    def getTileRect(self, value: int) -> Rect:
        if value not in self.__tilesRect:
            raise ValueError(f"No {value} in tileset {self.__imageFile}")
        return self.__tilesRect[value]

    def getTilesRect(self) -> Dict[int, Rect]:
        return self.__tilesRect

